package com.selesse.android.winedb.winescraper;

import android.app.Activity;
import android.app.ProgressDialog;
import android.os.AsyncTask;

import com.selesse.android.winedb.R;
import com.selesse.android.winedb.activity.WineDB;
import com.selesse.android.winedb.database.Wine;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public class WineScraperThread extends AsyncTask<String, Void, List<Wine>> {
    private ProgressDialog progress;
    private String barcode;
    private Activity activity;

    public WineScraperThread(Activity activity) {
        this.activity = activity;
    }

    @Override
    protected void onPreExecute() {
        super.onPreExecute();
        progress = new ProgressDialog(activity);
        try {
            progress.setMessage(activity.getString(R.string.scraping));
            progress.setIndeterminate(true);
            progress.show();
        } catch (Exception e) {
            // The fetch may happen too fast, ignore any exceptions that may arise due to this
        }
    }

    @Override
    protected List<Wine> doInBackground(String... params) {
        barcode = params[0];
        WineScrapers scrapers = new WineScrapers(barcode);
        return scrapers.scrape();
    }

    @Override
    protected void onPostExecute(List<Wine> result) {
        super.onPostExecute(result);

        try {
            progress.dismiss();
            progress = null;
        } catch (Exception e) {
            // do nothing
        }

        Wine wine;
        if (result.size() == 0) {
            wine = new Wine();
        }
        else {
            wine = result.get(0);
            if (result.size() > 1) {
                Collections.sort(result, new WineNumberOfFieldsComparator());
            }
        }
        wine.setBarcode(barcode);
        ((WineDB) activity).startEditWineIntent(wine);
    }

    /**
     * Wine Comparator that puts the wines with the most amount of fields first.
     */
    public static class WineNumberOfFieldsComparator implements Comparator<Wine> {
        @Override
        public int compare(Wine w1, Wine w2) {
            return w2.getNumberOfFilledFields() - w1.getNumberOfFilledFields();
        }
    }
}
